/******************************************************************************
* Copyright (c) 2019 - 2020 Xilinx, Inc.  All rights reserved.
* SPDX-License-Identifier: MIT
******************************************************************************/


/*****************************************************************************/
/**
*
* @file xsecure_aes.c
*
* This file contains AES hardware interface APIs
*
* <pre>
* MODIFICATION HISTORY:
*
* Ver   Who  Date        Changes
* ----- ---- ---------- -------------------------------------------------------
* 4.0   vns  04/24/2019 Initial release
* 4.1   vns  08/06/2019 Added AES encryption APIs
*       har  08/21/2019 Fixed MISRA C violations
*       vns  08/23/2019 Initialized status variables
* 4.2   har  01/03/2020 Added checks for return value of XSecure_SssAes
*       vns  02/10/2020 Added DPA CM enable/disable function
*       rpo  02/27/2020 Removed function prototype and static keyword of
*                       XSecure_AesKeyLoad, XSecure_AesWaitForDone functions
*       har  03/01/2020 Added code to soft reset once key decryption is done
*       rpo  03/23/2020 Replaced timeouts with WaitForEvent and code clean up
*       rpo  04/02/2020 Added Crypto KAT APIs
*                       Added support of release and set reset for AES
*       bvi  04/07/2020 Renamed csudma as pmcdma
*       vns  04/12/2020 Reset Versal key clear register in AES initialize call
* 4.3   ana  06/04/2020 Updated NextBlkLen in Xsecure_Aes structure wherever required
*                       Updated Aes state
*       kpt  06/29/2020 Added asserts for input arguments and minor
*                       enhancements on AES state
*       kpt  07/03/2020 Added type casting for the arguments in
*                       XPmcDma_64BitTransfer
*       kpt  07/08/2020 Removed dummy code and Status value reinitialized
*                       to XST_FAILURE
*       har  07/12/2020 Removed magic number from XSecure_AesKeyZero
*       har  07/21/2020 Corrected input parameters for config in
*                       XSecure_AesCfgKupIv
*       kpt  08/06/2020 Replaced magic numbers with macro's
* </pre>
*
* @note
*
******************************************************************************/

/***************************** Include Files *********************************/
#include "xsecure_utils.h"
#include "xsecure_aes.h"
#include "xsecure_aes_core_hw.h"
#include "xil_util.h"
#include "xsecure_error.h"
/************************** Constant Definitions *****************************/

#define XSECURE_MAX_KEY_SOURCES				XSECURE_AES_EXPANDED_KEYS
#define XSECURE_KEK_DEC_ENABLE                  (0x1U)
#define XSECURE_KEK_DEC_DISABLE                 (0x0U)
#define XSECURE_AES_DISABLE_KUP_IV_UPDATE		(0x0U)
#define XSECURE_AES_ENABLE_KUP_IV_UPDATE		(0x1U)
#define XSECURE_ENABLE_BYTE_SWAP				(0x1U)
#define XSECURE_DISABLE_BYTE_SWAP				(0x0U)
#define XSECURE_KAT_IV_SIZE_IN_WORDS			(4U)
#define XSECURE_KAT_MSG_SIZE_IN_WORDS			(4U)
#define XSECURE_KAT_GCMTAG_SIZE_IN_WORDS		(4U)
#define XSECURE_KAT_AES_SPLIT_DATA_SIZE			(4U)
#define XSECURE_KAT_KEY_SIZE_IN_WORDS			(8U)
#define XSECURE_KAT_OPER_DATA_SIZE_IN_WORDS		(16U)

/**************************** Type Definitions *******************************/
typedef struct {
	u32 RegOffset;
	u32 KeySrcSelVal;
	u8  UsrWrAllowed;
	u8  DecAllowed;
	u8  EncAllowed;
	u8  KeyDecSrcAllowed;
	u32 KeyDecSrcSelVal;
	u32 KeyClearVal;
} XSecure_AesKeyLookup;

/***************** Macros (Inline Functions) Definitions *********************/
/************************** Function Prototypes ******************************/

static u32 XSecure_AesWaitForDone(XSecure_Aes *InstancePtr);
static u32 XSecure_AesKeyLoad(XSecure_Aes *InstancePtr,
		XSecure_AesKeySrc KeySrc, XSecure_AesKeySize KeySize);
static void XSecure_AesPmcDmaCfgEndianness(XPmcDma *InstancePtr,
		XPmcDma_Channel Channel, u8 EndianType);
static u32 XSecure_AesKekWaitForDone(XSecure_Aes *InstancePtr);
static u32 XSecure_AesDpaCmDecryptKat(XSecure_Aes *AesInstance,
		const u32 *KeyPtr, const u32 *DataPtr, u32 *OutputPtr);
static u32 XSecure_AesEncNDecInit(XSecure_Aes *InstancePtr,
	XSecure_AesKeySrc KeySrc, XSecure_AesKeySize KeySize, u64 IvAddr);

/************************** Variable Definitions *****************************/
static const XSecure_AesKeyLookup AesKeyLookupTbl [XSECURE_MAX_KEY_SOURCES] =
{
	/* BBRAM_KEY */
	{ XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_SEL_BBRAM_KEY,
	  FALSE,
	  TRUE,
	  TRUE,
	  TRUE,
	  XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_INVALID_CFG
	},

	/* BBRAM_RED_KEY */
	{ XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_SEL_BBRAM_RD_KEY,
	  FALSE,
	  TRUE,
	  TRUE,
	  FALSE,
	  XSECURE_AES_KEY_DEC_SEL_BBRAM_RED,
	  XSECURE_AES_KEY_CLEAR_BBRAM_RED_KEY_MASK
	},

	/* BH_KEY */
	{ XSECURE_AES_BH_KEY_0_OFFSET,
	  XSECURE_AES_KEY_SEL_BH_KEY,
	  TRUE,
	  TRUE,
	  TRUE,
	  TRUE,
	  XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_CLEAR_BH_KEY_MASK
	},

	/* BH_RED_KEY */
	{ XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_SEL_BH_RD_KEY,
	  FALSE,
	  TRUE,
	  TRUE,
	  FALSE,
	  XSECURE_AES_KEY_DEC_SEL_BH_RED,
	  XSECURE_AES_KEY_CLEAR_BH_RED_KEY_MASK
	},

	/* EFUSE_KEY */
	{ XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_SEL_EFUSE_KEY,
	  FALSE,
	  TRUE,
	  TRUE,
	  TRUE,
	  XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_CLEAR_EFUSE_KEY_MASK
	},

	/* EFUSE_RED_KEY */
	{ XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_SEL_EFUSE_RED_KEY,
	  FALSE,
	  TRUE,
	  TRUE,
	  FALSE,
	  XSECURE_AES_KEY_DEC_SEL_EFUSE_RED,
	  XSECURE_AES_KEY_CLEAR_EFUSE_RED_KEY_MASK
	},

	/* EFUSE_USER_KEY_0 */
	{ XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_SEL_EFUSE_USR_KEY0,
	  FALSE,
	  TRUE,
	  TRUE,
	  TRUE,
	  XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_CLEAR_EFUSE_USER_KEY_0_MASK
	},

	/* EFUSE_USER_KEY_1 */
	{ XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_SEL_EFUSE_USR_KEY1,
	  FALSE,
	  TRUE,
	  TRUE,
	  TRUE,
	  XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_CLEAR_EFUSE_USER_KEY_1_MASK
	},

	/* EFUSE_USER_RED_KEY_0 */
	{ XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_SEL_EFUSE_USR_RD_KEY0,
	  FALSE,
	  TRUE,
	  TRUE,
	  FALSE,
	  XSECURE_AES_KEY_DEC_SEL_EFUSE_USR0_RED,
	  XSECURE_AES_KEY_CLEAR_EFUSE_USER_RED_KEY_0_MASK
	},

	/* EFUSE_USER_RED_KEY_1 */
	{ XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_SEL_EFUSE_USR_RD_KEY1,
	  FALSE,
	  TRUE,
	  TRUE,
	  FALSE,
	  XSECURE_AES_KEY_DEC_SEL_EFUSE_USR1_RED,
	  XSECURE_AES_KEY_CLEAR_EFUSE_USER_RED_KEY_1_MASK
	},

	/* KUP_KEY */
	{ XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_SEL_KUP_KEY,
	  FALSE,
	  TRUE,
	  TRUE,
	  FALSE,
	  XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_CLEAR_KUP_KEY_MASK
	},

	/* FAMILY_KEY */
	{ XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_SEL_FAMILY_KEY,
	  FALSE,
	  TRUE,
	  TRUE,
	  FALSE,
	  XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_INVALID_CFG
	},

	/* PUF_KEY */
	{ XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_SEL_PUF_KEY,
	  FALSE,
	  TRUE,
	  TRUE,
	  FALSE,
	  XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_CLEAR_PUF_KEY_MASK
	},

	/* USER_KEY_0 */
	{ XSECURE_AES_USER_KEY_0_0_OFFSET,
	  XSECURE_AES_KEY_SEL_USR_KEY_0,
	  TRUE,
	  TRUE,
	  TRUE,
	  FALSE,
	  XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_CLEAR_USER_KEY_0_MASK
	},

	/* USER_KEY_1 */
	{ XSECURE_AES_USER_KEY_1_0_OFFSET,
	  XSECURE_AES_KEY_SEL_USR_KEY_1,
	  TRUE,
	  TRUE,
	  TRUE,
	  FALSE,
	  XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_CLEAR_USER_KEY_1_MASK
	},

	/* USER_KEY_2 */
	{ XSECURE_AES_USER_KEY_2_0_OFFSET,
	  XSECURE_AES_KEY_SEL_USR_KEY_2,
	  TRUE,
	  TRUE,
	  TRUE,
	  FALSE,
	  XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_CLEAR_USER_KEY_2_MASK
	},

	/* USER_KEY_3 */
	{ XSECURE_AES_USER_KEY_3_0_OFFSET,
	  XSECURE_AES_KEY_SEL_USR_KEY_3,
	  TRUE,
	  TRUE,
	  TRUE,
	  FALSE,
	  XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_CLEAR_USER_KEY_3_MASK
	},

	/* USER_KEY_4 */
	{ XSECURE_AES_USER_KEY_4_0_OFFSET,
	  XSECURE_AES_KEY_SEL_USR_KEY_4,
	  TRUE,
	  TRUE,
	  TRUE,
	  FALSE,
	  XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_CLEAR_USER_KEY_4_MASK
	},

	/* USER_KEY_5 */
	{ XSECURE_AES_USER_KEY_5_0_OFFSET,
	  XSECURE_AES_KEY_SEL_USR_KEY_6,
	  TRUE,
	  TRUE,
	  TRUE,
	  FALSE,
	  XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_CLEAR_USER_KEY_5_MASK
	},

	/* USER_KEY_6 */
	{ XSECURE_AES_USER_KEY_6_0_OFFSET,
	  XSECURE_AES_KEY_SEL_USR_KEY_6,
	  TRUE,
	  TRUE,
	  TRUE,
	  FALSE,
	  XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_CLEAR_USER_KEY_6_MASK
	},

	/* USER_KEY_7 */
	{ XSECURE_AES_USER_KEY_7_0_OFFSET,
	  XSECURE_AES_KEY_SEL_USR_KEY_7,
	  TRUE,
	  TRUE,
	  TRUE,
	  FALSE,
	  XSECURE_AES_INVALID_CFG,
	  XSECURE_AES_KEY_CLEAR_USER_KEY_7_MASK
	}
};

/*****************************************************************************/
/**
 * @brief
 * This function initializes the instance pointer.
 *
 * @param	InstancePtr	Pointer to the XSecure_Aes instance.
 * @param	PmcDmaPtr	Pointer to the XPmcDma instance.
 *
 * @return	XST_SUCCESS if initialization was successful.
 *
 * @note	All the inputs are accepted in little endian format, but AES
 *		engine accepts the data in big endianness, this will be taken
 *		care while passing data to AES engine.
 *
 ******************************************************************************/
u32 XSecure_AesInitialize(XSecure_Aes *InstancePtr, XPmcDma *PmcDmaPtr)
{
	u32 Status = (u32)XST_FAILURE;

	Xil_AssertNonvoid(InstancePtr != NULL);
	Xil_AssertNonvoid(PmcDmaPtr != NULL);

	/* Initialize the instance */
	InstancePtr->BaseAddress = XSECURE_AES_BASEADDR;
	InstancePtr->PmcDmaPtr = PmcDmaPtr;
	InstancePtr->NextBlkLen = 0U;

	/* Clear all key zeroization register */
	XSecure_WriteReg(InstancePtr->BaseAddress,
		XSECURE_AES_KEY_CLEAR_OFFSET, XSECURE_AES_KEY_CLEAR_ALL_AES_KEYS);

	XSecure_SssInitialize(&(InstancePtr->SssInstance));

	InstancePtr->AesState = XSECURE_AES_INITIALIZED;

	Status = XST_SUCCESS;

	return Status;
}

/*****************************************************************************/
/**
 * @brief
 * This function enables or disables DPA counter measures in AES engine based
 * on user input.
 *
 * @param	InstancePtr	Pointer to the XSecure_Aes instance.
 * @param	DpaCmCfg	User choice to enable/disable DPA CM.
 *			- TRUE - to enable AES DPA counter measure (Default setting)
 *			- FALSE - to disable AES DPA counter measure
 *
 * @return
 *			- XSECURE_AES_DPA_CM_NOT_SUPPORTED if DPA CM is disabled on chip.
 *			(Enabling/Disabling in AES engine does not impact functionality)
 *			- XST_SUCCESS if configuration is success.
 *
 ******************************************************************************/
u32 XSecure_AesSetDpaCm(XSecure_Aes *InstancePtr, u32 DpaCmCfg)
{
	u32 Status = (u32)XST_FAILURE;
	u32 ReadReg;

	Xil_AssertNonvoid(InstancePtr != NULL);
	Xil_AssertNonvoid((DpaCmCfg == TRUE) ||
					(DpaCmCfg == FALSE));
	Xil_AssertNonvoid(InstancePtr->AesState != XSECURE_AES_UNINITIALIZED);

	/* Chip has DPA CM support */
	if ((XSecure_In32(XSECURE_EFUSE_SECURITY_MISC1) &
		XSECURE_EFUSE_DPA_CM_DIS_MASK) == 0x00U) {

		/* Disable/enable DPA CM inside AES engine */
		XSecure_WriteReg(InstancePtr->BaseAddress,
						XSECURE_AES_CM_EN_OFFSET, DpaCmCfg);

		/* Verify status of CM */
		ReadReg = XSecure_ReadReg(InstancePtr->BaseAddress,
				XSECURE_AES_STATUS_OFFSET);
		ReadReg = (ReadReg & XSECURE_AES_STATUS_CM_ENABLED_MASK) >>
					XSECURE_AES_STATUS_CM_ENABLED_SHIFT;
		if (ReadReg == DpaCmCfg) {
			Status = (u32)XST_SUCCESS;
		}
	}
	else {
		Status = (u32)XSECURE_AES_DPA_CM_NOT_SUPPORTED;
	}

	return Status;
}
/*****************************************************************************/
/**
 *
 * @brief
 * This function writes the key provided into the specified AES key registers.
 *
 * @param	InstancePtr	Pointer to the XSecure_Aes instance.
 * @param	KeySrc		Key Source to be selected to which provided
 *		key should be updated.
 *		- XSECURE_AES_USER_KEY_0
 *		- XSECURE_AES_USER_KEY_1
 *		- XSECURE_AES_USER_KEY_2
 *		- XSECURE_AES_USER_KEY_3
 *		- XSECURE_AES_USER_KEY_4
 *		- XSECURE_AES_USER_KEY_5
 *		- XSECURE_AES_USER_KEY_6
 *		- XSECURE_AES_USER_KEY_7
 *		- XSECURE_AES_BH_KEY
 * @param	KeySize		A variable of type XSecure_AesKeySize, which
 *		holds the size of the input key to be written.
 *		 - XSECURE_AES_KEY_SIZE_128 for 128 bit key size
 *		 - XSECURE_AES_KEY_SIZE_256 for 256 bit key size
 * @param	KeyAddr		Address of a buffer which should contain
 *		the key to be written.
 *
 * @return
 *		- XST_SUCCESS on successful written
 *		- Error code on failure
 *
 ******************************************************************************/
u32 XSecure_AesWriteKey(XSecure_Aes *InstancePtr, XSecure_AesKeySrc KeySrc,
			XSecure_AesKeySize KeySize, u64 KeyAddr)
{
	u32 Offset;
	u32 Index = 0U;
	u32 *Key;
	u32 KeySizeInWords;
	u32 Status = (u32)XST_FAILURE;

	Xil_AssertNonvoid(InstancePtr != NULL);
	Xil_AssertNonvoid((KeySrc < XSECURE_MAX_KEY_SOURCES) &&
		(KeySrc >= XSECURE_AES_BBRAM_KEY));
	Xil_AssertNonvoid(AesKeyLookupTbl[KeySrc].UsrWrAllowed == TRUE);
	Xil_AssertNonvoid((XSECURE_AES_KEY_SIZE_128 == KeySize) ||
			(XSECURE_AES_KEY_SIZE_256 == KeySize));
	Xil_AssertNonvoid(KeyAddr != 0x00U);
	Xil_AssertNonvoid(InstancePtr->AesState != XSECURE_AES_UNINITIALIZED);

	Key = (u32 *)(INTPTR)KeyAddr;


	if ((XSECURE_AES_BH_KEY == KeySrc) &&
			(XSECURE_AES_KEY_SIZE_128 == KeySize)) {
		Status = XST_FAILURE;
		goto END;
	}

	if (XSECURE_AES_KEY_SIZE_128 == KeySize) {
		KeySizeInWords = XSECURE_AES_KEY_SIZE_128BIT_WORDS;
	}
	else {
		KeySizeInWords = XSECURE_AES_KEY_SIZE_256BIT_WORDS;
	}

	Offset = AesKeyLookupTbl[KeySrc].RegOffset;
	if (Offset == XSECURE_AES_INVALID_CFG) {
		Status = XST_FAILURE;
		goto END;
	}

	Offset = Offset + (KeySizeInWords * XSECURE_WORD_SIZE) -
				XSECURE_WORD_SIZE;

	for (Index = 0U; Index < KeySizeInWords; Index++) {
		XSecure_WriteReg(InstancePtr->BaseAddress, Offset,
					Xil_Htonl(Key[Index]));
		Offset = Offset - XSECURE_WORD_SIZE;
	}
	Status = XST_SUCCESS;

END:
	return Status;
}


/*****************************************************************************/
/**
 * This function decrypts the key which is in KEK/Obfuscated key form and
 * exist in either of the boot header/Efuse/BBRAM and updates the mentioned
 * destination red key register with corresponding red key.
 *
 * @param	InstancePtr	Pointer to the XSecure_Aes instance.
 * @param	KeyType		The source of key to be used for decryption
 * 			- XSECURE_BLACK_KEY
 * 			- XSECURE_OBFUSCATED_KEY
 * @param	DecKeySrc	Select key source which holds KEK and needs to be
 *		decrypted
 * @param	DstKeySrc	Select the key in which decrypted red key should be
 *		updated
 * @param	IvAddr		Address of IV holding buffer for decryption
 *		of the key.
 * @param	KeySize		A variable of type XSecure_AesKeySize, which
 *		specifies the size of the key.
 *		 - XSECURE_AES_KEY_SIZE_128 for 128 bit key size
 *		 - XSECURE_AES_KEY_SIZE_256 for 256 bit key size
 *
 * @return
 *		- XST_SUCCESS on successful key decryption.
 *		- Error code on failure
 *
 ******************************************************************************/
u32 XSecure_AesKekDecrypt(XSecure_Aes *InstancePtr, XSecure_AesKekType KeyType,
			  XSecure_AesKeySrc DecKeySrc,
			  XSecure_AesKeySrc DstKeySrc, u64 IvAddr, u32 KeySize)
{
	u32 Status = (u32)XST_FAILURE;
	XSecure_AesKeySrc KeySrc;

	Xil_AssertNonvoid(InstancePtr != NULL);
	Xil_AssertNonvoid((KeyType == XSECURE_BLACK_KEY) ||
			(KeyType == XSECURE_OBFUSCATED_KEY));
	Xil_AssertNonvoid((DecKeySrc < XSECURE_MAX_KEY_SOURCES) &&
		(DecKeySrc >= XSECURE_AES_BBRAM_KEY));
	Xil_AssertNonvoid((DstKeySrc < XSECURE_MAX_KEY_SOURCES) &&
		(DstKeySrc >= XSECURE_AES_BBRAM_KEY));
	Xil_AssertNonvoid(IvAddr != 0x00U);
	Xil_AssertNonvoid((KeySize == XSECURE_AES_KEY_SIZE_128) ||
		(KeySize == XSECURE_AES_KEY_SIZE_256));



	if ((AesKeyLookupTbl[DecKeySrc].KeyDecSrcAllowed != TRUE) ||
	    (AesKeyLookupTbl[DstKeySrc].KeyDecSrcSelVal ==
				XSECURE_AES_INVALID_CFG)) {
		Status = XST_INVALID_PARAM;
		goto END;
	}

	XSecure_ReleaseReset(InstancePtr->BaseAddress,
		XSECURE_AES_SOFT_RST_OFFSET);

	/* Configure the SSS for AES. */
	if (InstancePtr->PmcDmaPtr->Config.DeviceId == PMCDMA_0_DEVICE_ID) {
		Status = XSecure_SssAes(&InstancePtr->SssInstance,
			XSECURE_SSS_DMA0, XSECURE_SSS_DMA0);
	}
	else {
		Status = XSecure_SssAes(&InstancePtr->SssInstance,
			XSECURE_SSS_DMA1, XSECURE_SSS_DMA1);
	}
	if (Status != (u32)XST_SUCCESS) {
		goto END;
	}


	if (KeyType == XSECURE_OBFUSCATED_KEY) {
		KeySrc = XSECURE_AES_FAMILY_KEY;
	}
	else if (KeyType == XSECURE_BLACK_KEY) {
		KeySrc = XSECURE_AES_PUF_KEY;
	}
	else {
		Status = XST_FAILURE;
		goto END;
	}

	/* Status Reset*/
	Status = (u32)XST_FAILURE;

	Status = XSecure_AesKeyLoad(InstancePtr, KeySrc, KeySize);
	if (Status != (u32)XST_SUCCESS) {
		goto END;
	}

	/* Start the message. */
	XSecure_WriteReg(InstancePtr->BaseAddress,
			XSECURE_AES_START_MSG_OFFSET,
			XSECURE_AES_START_MSG_VAL_MASK);

	/* Enable PMC DMA Src channel for byte swapping.*/
	XSecure_AesPmcDmaCfgEndianness(InstancePtr->PmcDmaPtr,
			XPMCDMA_SRC_CHANNEL, XSECURE_ENABLE_BYTE_SWAP);

	XSecure_WriteReg(InstancePtr->BaseAddress,
		XSECURE_AES_DATA_SWAP_OFFSET, XSECURE_ENABLE_BYTE_SWAP);

	/* Push IV into the AES engine. */
	XPmcDma_64BitTransfer(InstancePtr->PmcDmaPtr, XPMCDMA_SRC_CHANNEL,
		(u32)IvAddr, (u32)(IvAddr >> 32U),
		XSECURE_SECURE_GCM_TAG_SIZE / XSECURE_WORD_SIZE, TRUE);

	XPmcDma_WaitForDone(InstancePtr->PmcDmaPtr, XPMCDMA_SRC_CHANNEL);


	XSecure_WriteReg(InstancePtr->BaseAddress,
			XSECURE_AES_DATA_SWAP_OFFSET, XSECURE_DISABLE_BYTE_SWAP);

	/* Acknowledge the transfer has completed */
	XPmcDma_IntrClear(InstancePtr->PmcDmaPtr, XPMCDMA_SRC_CHANNEL,
				XPMCDMA_IXR_DONE_MASK);

	/* Select key decryption */
	XSecure_WriteReg(InstancePtr->BaseAddress,
		XSECURE_AES_KEY_DEC_OFFSET, XSECURE_AES_KEY_DEC_MASK);

	/* Decrypt selection */
	XSecure_WriteReg(InstancePtr->BaseAddress,
		XSECURE_AES_KEY_DEC_SEL_OFFSET,
		AesKeyLookupTbl[DstKeySrc].KeyDecSrcSelVal);

	XSecure_WriteReg(InstancePtr->BaseAddress,
		XSECURE_AES_KEY_SEL_OFFSET,
		AesKeyLookupTbl[DecKeySrc].KeySrcSelVal);

	XSecure_WriteReg(InstancePtr->BaseAddress,
			XSECURE_AES_KEY_DEC_TRIG_OFFSET, XSECURE_KEK_DEC_ENABLE);

	/* Status Reset*/
	Status = (u32)XST_FAILURE;

	/* Wait for AES Decryption completion. */
	Status = XSecure_AesKekWaitForDone(InstancePtr);
END:
	XSecure_SetReset(InstancePtr->BaseAddress,
		XSECURE_AES_SOFT_RST_OFFSET);

	/* Select key decryption */
	XSecure_WriteReg(InstancePtr->BaseAddress,
			XSECURE_AES_KEY_DEC_OFFSET, XSECURE_AES_KEY_DEC_RESET_MASK);

	return Status;
}
/*****************************************************************************/
/**
 *
 * @brief
 * This function initializes the AES engine for decryption.
 *
 * @param	InstancePtr	Pointer to the XSecure_Aes instance.
 * @param	KeySrc		Key Source for decryption of the data.
 * @param	KeySize		Size of the AES key to be used for decryption.
 *		 - XSECURE_AES_KEY_SIZE_128 for 128 bit key size
 *		 - XSECURE_AES_KEY_SIZE_256 for 256 bit key size
 * @param	IvAddr		Address to the buffer holding IV.
 *
 * @return
 *		- XST_SUCCESS on successful init
 *		- Error code on failure
 *
 ******************************************************************************/
u32 XSecure_AesDecryptInit(XSecure_Aes *InstancePtr, XSecure_AesKeySrc KeySrc,
			XSecure_AesKeySize KeySize, u64 IvAddr)
{
	u32 Status = (u32)XST_FAILURE;

	/* Assert validates the input arguments */
	Xil_AssertNonvoid(InstancePtr != NULL);
	Xil_AssertNonvoid((KeySrc < XSECURE_MAX_KEY_SOURCES) &&
		(KeySrc >= XSECURE_AES_BBRAM_KEY));
	Xil_AssertNonvoid((KeySize == XSECURE_AES_KEY_SIZE_128) ||
		(KeySize == XSECURE_AES_KEY_SIZE_256));
	Xil_AssertNonvoid(IvAddr != 0x00U);
	Xil_AssertNonvoid(InstancePtr->AesState != XSECURE_AES_UNINITIALIZED);

	if(InstancePtr->NextBlkLen == 0U) {
		XSecure_ReleaseReset(InstancePtr->BaseAddress,
			XSECURE_AES_SOFT_RST_OFFSET);
	}

	/* Key selected does not allow decryption */
	if (AesKeyLookupTbl[KeySrc].DecAllowed == FALSE) {
		Status = XST_FAILURE;
		goto END;
	}

	/* Configure AES for decryption */
	XSecure_WriteReg(InstancePtr->BaseAddress,
			XSECURE_AES_MODE_OFFSET, XSECURE_AES_MODE_DEC);

	Status = XSecure_AesEncNDecInit(InstancePtr, KeySrc, KeySize, IvAddr);
	if (Status != (u32)XST_SUCCESS) {
		goto END;
	}
	/* Update the state */
	InstancePtr->AesState = XSECURE_AES_DECRYPT_INITIALIZED;

	Status = XST_SUCCESS;

END:
	if (Status != (u32)XST_SUCCESS) {
		/*
		 * To make sure that Decrypt init function
		 * placing AES engine into reset on failure
		 */
		InstancePtr->NextBlkLen = 0U;
		InstancePtr->AesState = XSECURE_AES_INITIALIZED;
		XSecure_SetReset(InstancePtr->BaseAddress,
			XSECURE_AES_SOFT_RST_OFFSET);
	}

	return Status;

}

/*****************************************************************************/
/**
 * @brief
 * This function is used to update the AES engine for decryption with provided
 * data and stores the decrypted data at specified address.
 *
 * @param	InstancePtr	Pointer to the XSecure_Aes instance.
 * @param	InDataAddr	Address of the encrypted data which needs to be
 *		decrypted.
 * @param	OutDataAddr	Address of output buffer where the decrypted
 *		to be updated.
 * @param	Size		Size of data to be decrypted in bytes,
 *			whereas number of bytes provided should be multiples of 4.
 * @param	IsLastChunk	If this is the last update of data to be
 *		decrypted, this parameter should be set to TRUE otherwise FALSE.
 *
 * @return
 *		- XST_SUCCESS on successful decryption of the data.
 *		- Error code on failure
 *
 ******************************************************************************/
u32 XSecure_AesDecryptUpdate(XSecure_Aes *InstancePtr, u64 InDataAddr,
		u64 OutDataAddr, u32 Size, u8 IsLastChunk)
{
	u32 Status = (u32)XST_FAILURE;

	/* Assert validates the input arguments */
	Xil_AssertNonvoid(InstancePtr != NULL);
	Xil_AssertNonvoid((InDataAddr != 0x00U) && (OutDataAddr != 0x00U));
	Xil_AssertNonvoid((Size % XSECURE_WORD_SIZE) == 0x00U);
	Xil_AssertNonvoid((IsLastChunk == TRUE) || (IsLastChunk == FALSE));
	Xil_AssertNonvoid(InstancePtr->AesState ==
			XSECURE_AES_DECRYPT_INITIALIZED);

	/* Enable PMC DMA Src and Dst channels for byte swapping.*/
	XSecure_AesPmcDmaCfgEndianness(InstancePtr->PmcDmaPtr,
				XPMCDMA_SRC_CHANNEL, XSECURE_ENABLE_BYTE_SWAP);

	if ((u32)OutDataAddr != XSECURE_AES_NO_CFG_DST_DMA) {
		XSecure_AesPmcDmaCfgEndianness(InstancePtr->PmcDmaPtr,
				XPMCDMA_DST_CHANNEL, XSECURE_ENABLE_BYTE_SWAP);
	}

	/* Configure the SSS for AES. */
	if (InstancePtr->PmcDmaPtr->Config.DeviceId == PMCDMA_0_DEVICE_ID) {
		Status = XSecure_SssAes(&InstancePtr->SssInstance,
			XSECURE_SSS_DMA0, XSECURE_SSS_DMA0);
	}
	else {
		Status = XSecure_SssAes(&InstancePtr->SssInstance,
			XSECURE_SSS_DMA1, XSECURE_SSS_DMA1);
	}
	if (Status != (u32)XST_SUCCESS) {
		goto END;
	}
	/* Configure destination */
	if ((u32)OutDataAddr != XSECURE_AES_NO_CFG_DST_DMA) {
		XPmcDma_64BitTransfer(InstancePtr->PmcDmaPtr,
				XPMCDMA_DST_CHANNEL,
				(u32)OutDataAddr, (u32)(OutDataAddr >> 32U),
				Size / XSECURE_WORD_SIZE, FALSE);
	}

	XPmcDma_64BitTransfer(InstancePtr->PmcDmaPtr,
				XPMCDMA_SRC_CHANNEL,
				(u32)InDataAddr, (u32)(InDataAddr >> 32U),
				Size / XSECURE_WORD_SIZE, IsLastChunk);

	/* Wait for the SRC DMA completion. */
	XPmcDma_WaitForDone(InstancePtr->PmcDmaPtr, XPMCDMA_SRC_CHANNEL);

	/* Acknowledge the transfer has completed */
	XPmcDma_IntrClear(InstancePtr->PmcDmaPtr, XPMCDMA_SRC_CHANNEL,
					XPMCDMA_IXR_DONE_MASK);

	if ((u32)OutDataAddr != XSECURE_AES_NO_CFG_DST_DMA) {
		/* Wait for the DST DMA completion. */
		XPmcDma_WaitForDone(InstancePtr->PmcDmaPtr, XPMCDMA_DST_CHANNEL);

		/* Acknowledge the transfer has completed */
		XPmcDma_IntrClear(InstancePtr->PmcDmaPtr, XPMCDMA_DST_CHANNEL,
						XPMCDMA_IXR_DONE_MASK);
	}

	Status = (u32)XST_SUCCESS;

END:
	/* Clear endianness */
	XSecure_AesPmcDmaCfgEndianness(InstancePtr->PmcDmaPtr,
				XPMCDMA_SRC_CHANNEL, XSECURE_DISABLE_BYTE_SWAP);
	XSecure_AesPmcDmaCfgEndianness(InstancePtr->PmcDmaPtr,
				XPMCDMA_DST_CHANNEL, XSECURE_DISABLE_BYTE_SWAP);
	if (Status != (u32)XST_SUCCESS) {
		/*
		 * To make sure that Decrypt update function
		 * placing AES engine into reset on failure
		 */
		InstancePtr->NextBlkLen = 0U;
		InstancePtr->AesState = XSECURE_AES_INITIALIZED;
		XSecure_SetReset(InstancePtr->BaseAddress,
			XSECURE_AES_SOFT_RST_OFFSET);
	}

	return Status;

}

/*****************************************************************************/
/**
 *
 * @brief
 * This function verifies the GCM tag provided for the data decrypted
 * till the point.
 *
 * @param	InstancePtr	Pointer to the XSecure_Aes instance.

 * @param	GcmTagAddr	Address of a buffer which should holds
 *		GCM Tag.
 *
 * @return
 *		- XST_SUCCESS on successful GCM tag verification
 *		- Error code on failure
 *
 ******************************************************************************/
u32 XSecure_AesDecryptFinal(XSecure_Aes *InstancePtr, u64 GcmTagAddr)
{
	u32 Status = (u32)XST_FAILURE;

	/* Assert validates the input arguments */
	Xil_AssertNonvoid(InstancePtr != NULL);
	Xil_AssertNonvoid(GcmTagAddr != 0x00U);
	Xil_AssertNonvoid(InstancePtr->AesState ==
			XSECURE_AES_DECRYPT_INITIALIZED);

	XSecure_WriteReg(InstancePtr->BaseAddress,
			XSECURE_AES_DATA_SWAP_OFFSET, XSECURE_ENABLE_BYTE_SWAP);

	XSecure_AesPmcDmaCfgEndianness(InstancePtr->PmcDmaPtr,
					XPMCDMA_SRC_CHANNEL, XSECURE_ENABLE_BYTE_SWAP);

	/* Configure the SSS for AES. */
	if (InstancePtr->PmcDmaPtr->Config.DeviceId == PMCDMA_0_DEVICE_ID) {
		Status = XSecure_SssAes(&InstancePtr->SssInstance,
				XSECURE_SSS_DMA0, XSECURE_SSS_DMA0);
	}
	else {
		Status = XSecure_SssAes(&InstancePtr->SssInstance,
				XSECURE_SSS_DMA1, XSECURE_SSS_DMA1);
	}
	if (Status != (u32)XST_SUCCESS) {
		goto END;
	}

	XPmcDma_64BitTransfer(InstancePtr->PmcDmaPtr,
		XPMCDMA_SRC_CHANNEL, (u32)GcmTagAddr, (u32)(GcmTagAddr >> 32U),
		XSECURE_SECURE_GCM_TAG_SIZE / XSECURE_WORD_SIZE, FALSE);

	/* Wait for the Src DMA completion. */
	XPmcDma_WaitForDone(InstancePtr->PmcDmaPtr, XPMCDMA_SRC_CHANNEL);

	/* Acknowledge the transfer has completed */
	XPmcDma_IntrClear(InstancePtr->PmcDmaPtr,
			XPMCDMA_SRC_CHANNEL,
			XPMCDMA_IXR_DONE_MASK);

	/* Status Reset*/
	Status = (u32)XST_FAILURE;

	/* Wait for AES Decryption completion. */
	Status = XSecure_AesWaitForDone(InstancePtr);
	if (Status != XST_SUCCESS) {
		goto END;
	}

	XSecure_AesPmcDmaCfgEndianness(InstancePtr->PmcDmaPtr,
					XPMCDMA_SRC_CHANNEL, XSECURE_DISABLE_BYTE_SWAP);

	/* Status Reset */
	Status = (u32)XST_FAILURE;

	/* Get the AES status to know if GCM check passed. */
	Status = XSecure_ReadReg(InstancePtr->BaseAddress,
			XSECURE_AES_STATUS_OFFSET);
	Status &= XSECURE_AES_STATUS_GCM_TAG_PASS_MASK;
	if (Status != XSECURE_AES_STATUS_GCM_TAG_PASS_MASK) {
		Status = XSECURE_AES_GCM_TAG_MISMATCH;
		goto END;
	}

	/* Status Reset */
	Status = (u32)XST_FAILURE;

	Status = XSecure_AesGetNxtBlkLen(InstancePtr, &InstancePtr->NextBlkLen);
END:
	XSecure_WriteReg(InstancePtr->BaseAddress,
			XSECURE_AES_DATA_SWAP_OFFSET, XSECURE_DISABLE_BYTE_SWAP);
	InstancePtr->AesState = XSECURE_AES_INITIALIZED;
	if ((InstancePtr->NextBlkLen == 0U) || (Status != (u32)XST_SUCCESS)) {
		/*
		 * To make sure that Decrypt final function
		 * placing AES engine into reset on failure
		 */
		InstancePtr->NextBlkLen = 0U;
		XSecure_SetReset(InstancePtr->BaseAddress,
			XSECURE_AES_SOFT_RST_OFFSET);
	}

	return Status;

}

/*****************************************************************************/
/**
 *
 * @brief
 * This function decrypts the  size (length) number of bytes of the passed in
 * InDataAddr (source) buffer and stores the decrypted data in the OutDataAddr
 * (destination) buffer and verifies GcmTagAddr.
 *
 * @param	InstancePtr	Pointer to the XSecure_Aes instance.
 * @param	InDataAddr	Address of the encrypted data which needs to be
 *		decrypted.
 * @param	OutDataAddr	Address of output buffer where the decrypted
 *		to be updated.
 * @param	Size		Size of data to be decrypted in bytes,
 *			whereas number of bytes provided should be multiples of 4.
 * @param	GcmTagAddr	Address of a buffer which should contain
 *		GCM Tag.
 *
 * @return
 *		- XST_SUCCESS on successful GCM tag verification
 *		- Error code on failure
 *
 ******************************************************************************/
u32 XSecure_AesDecryptData(XSecure_Aes *InstancePtr, u64 InDataAddr,
			u64 OutDataAddr, u32 Size, u64 GcmTagAddr)
{
	u32 Status = (u32)XST_FAILURE;

	/* Assert validates the input arguments */
	Xil_AssertNonvoid(InstancePtr != NULL);
	Xil_AssertNonvoid((InDataAddr != 0x00U) && (OutDataAddr != 0x00U));
	Xil_AssertNonvoid((Size % XSECURE_WORD_SIZE) == 0x00U);
	Xil_AssertNonvoid(GcmTagAddr != 0x00U);
	Xil_AssertNonvoid(InstancePtr->AesState ==
			XSECURE_AES_DECRYPT_INITIALIZED);

	/* Update AES engine with data */
	Status = XSecure_AesDecryptUpdate(InstancePtr, InDataAddr, OutDataAddr,
						Size, TRUE);
	if (Status != XST_SUCCESS) {
		goto END;
	}

	/* Status Reset */
	Status = (u32)XST_FAILURE;

	/* Verify GCM tag */
	Status = XSecure_AesDecryptFinal(InstancePtr, GcmTagAddr);
END:
	return Status;
}

/*****************************************************************************/
/**
 *
 * @brief
 * This function initializes the AES engine for encryption.
 *
 * @param	InstancePtr	Pointer to the XSecure_Aes instance.
 * @param	KeySrc		Key Source for encryption.
 * @param	KeySize		Size of the AES key to be used for encryption.
 *			 - XSECURE_AES_KEY_SIZE_128 for 128 bit key size
 *			 - XSECURE_AES_KEY_SIZE_256 for 256 bit key size
 * @param	IvAddr		Address to the buffer holding IV.
 *
 * @return
 *		- XST_SUCCESS on successful init
 *		- Error code on failure
 *
 ******************************************************************************/
u32 XSecure_AesEncryptInit(XSecure_Aes *InstancePtr, XSecure_AesKeySrc KeySrc,
			XSecure_AesKeySize KeySize, u64 IvAddr)
{

	u32 Status = (u32)XST_FAILURE;

	/* Assert validates the input arguments */
	Xil_AssertNonvoid(InstancePtr != NULL);
	Xil_AssertNonvoid((KeySrc < XSECURE_MAX_KEY_SOURCES) &&
		(KeySrc >= XSECURE_AES_BBRAM_KEY));
	Xil_AssertNonvoid((KeySize == XSECURE_AES_KEY_SIZE_128) ||
		(KeySize == XSECURE_AES_KEY_SIZE_256));
	Xil_AssertNonvoid(IvAddr != 0x00U);
	Xil_AssertNonvoid(InstancePtr->AesState != XSECURE_AES_UNINITIALIZED);

	XSecure_ReleaseReset(InstancePtr->BaseAddress,
				XSECURE_AES_SOFT_RST_OFFSET);

	/* Key selected does not allow Encryption */
	if (AesKeyLookupTbl[KeySrc].EncAllowed == FALSE) {
		Status = XST_FAILURE;
		goto END;
	}

	/* Configure AES for Encryption */
	XSecure_WriteReg(InstancePtr->BaseAddress,
			XSECURE_AES_MODE_OFFSET, XSECURE_AES_MODE_ENC);

	Status = XSecure_AesEncNDecInit(InstancePtr, KeySrc, KeySize, IvAddr);
	if(Status != XST_SUCCESS) {
		goto END;
	}
	/* Update the state */
	InstancePtr->AesState = XSECURE_AES_ENCRYPT_INITIALIZED;

END:
	if (Status != (u32)XST_SUCCESS) {
		InstancePtr->AesState = XSECURE_AES_INITIALIZED;
		XSecure_SetReset(InstancePtr->BaseAddress,
			XSECURE_AES_SOFT_RST_OFFSET);
	}

	return Status;
}

/*****************************************************************************/
/**
 * @brief
 * This function is used to update the AES engine for encryption with provided
 * data and stores the decrypted data at specified address.
 *
 * @param	InstancePtr	Pointer to the XSecure_Aes instance.
 * @param	InDataAddr	Address of the encrypted data which needs to be
 *		encrypted.
 * @param	OutDataAddr	Address of output buffer where the encrypted data
 *		to be updated.
 * @param	Size		Size of data to be encrypted in bytes,
 *			whereas number of bytes provided should be multiples of 4.
 * @param	IsLastChunk		If this is the last update of data to be
 *		encrypted, this parameter should be set to TRUE otherwise FALSE.
 *
 * @return
 *		- XST_SUCCESS on successful encryption of the data.
 *		- Error code on failure
 *
 ******************************************************************************/
u32 XSecure_AesEncryptUpdate(XSecure_Aes *InstancePtr, u64 InDataAddr,
			u64 OutDataAddr, u32 Size, u8 IsLastChunk)
{
	u32 Status = (u32)XST_FAILURE;

	/* Assert validates the input arguments */
	Xil_AssertNonvoid(InstancePtr != NULL);
	Xil_AssertNonvoid((InDataAddr != 0x00U) && (OutDataAddr != 0x00U));
	Xil_AssertNonvoid((Size % XSECURE_WORD_SIZE) == 0x00U);
	Xil_AssertNonvoid((IsLastChunk == TRUE) || (IsLastChunk == FALSE));
	Xil_AssertNonvoid(InstancePtr->AesState ==
			XSECURE_AES_ENCRYPT_INITIALIZED);

	/* Enable PMC DMA Src and Dst channels for byte swapping.*/
	XSecure_AesPmcDmaCfgEndianness(InstancePtr->PmcDmaPtr,
				XPMCDMA_SRC_CHANNEL, XSECURE_ENABLE_BYTE_SWAP);

	if ((u32)OutDataAddr != XSECURE_AES_NO_CFG_DST_DMA) {
		XSecure_AesPmcDmaCfgEndianness(InstancePtr->PmcDmaPtr,
				XPMCDMA_DST_CHANNEL, XSECURE_ENABLE_BYTE_SWAP);
	}

	/* Configure the SSS for AES. */
	if (InstancePtr->PmcDmaPtr->Config.DeviceId == PMCDMA_0_DEVICE_ID) {
		Status = XSecure_SssAes(&InstancePtr->SssInstance,
			XSECURE_SSS_DMA0, XSECURE_SSS_DMA0);
	}
	else {
		Status = XSecure_SssAes(&InstancePtr->SssInstance,
			XSECURE_SSS_DMA1, XSECURE_SSS_DMA1);
	}
	if (Status != (u32)XST_SUCCESS) {
		goto END;
	}
	/* Configure destination */
	if ((u32)OutDataAddr != XSECURE_AES_NO_CFG_DST_DMA) {
		XPmcDma_64BitTransfer(InstancePtr->PmcDmaPtr,
						XPMCDMA_DST_CHANNEL,
						(u32)OutDataAddr, (u32)(OutDataAddr >> 32U),
						Size / XSECURE_WORD_SIZE, FALSE);
	}

	XPmcDma_64BitTransfer(InstancePtr->PmcDmaPtr,
				XPMCDMA_SRC_CHANNEL,
				(u32)InDataAddr, (u32)(InDataAddr >> 32U),
				Size / XSECURE_WORD_SIZE, IsLastChunk);

	/* Wait for the SRC DMA completion. */
	XPmcDma_WaitForDone(InstancePtr->PmcDmaPtr, XPMCDMA_SRC_CHANNEL);

	/* Acknowledge the transfer has completed */
	XPmcDma_IntrClear(InstancePtr->PmcDmaPtr, XPMCDMA_SRC_CHANNEL,
					XPMCDMA_IXR_DONE_MASK);

	if ((u32)OutDataAddr != XSECURE_AES_NO_CFG_DST_DMA) {
		/* Wait for the DST DMA completion. */
		XPmcDma_WaitForDone(InstancePtr->PmcDmaPtr,
				XPMCDMA_DST_CHANNEL);

		/* Acknowledge the transfer has completed */
		XPmcDma_IntrClear(InstancePtr->PmcDmaPtr,
				XPMCDMA_DST_CHANNEL,
				XPMCDMA_IXR_DONE_MASK);
	}

	Status = (u32)XST_SUCCESS;

END:
	/* Clear endianness */
	XSecure_AesPmcDmaCfgEndianness(InstancePtr->PmcDmaPtr,
				XPMCDMA_SRC_CHANNEL, XSECURE_DISABLE_BYTE_SWAP);
	XSecure_AesPmcDmaCfgEndianness(InstancePtr->PmcDmaPtr,
				XPMCDMA_DST_CHANNEL, XSECURE_DISABLE_BYTE_SWAP);
	if (Status != (u32)XST_SUCCESS) {
		InstancePtr->AesState = XSECURE_AES_INITIALIZED;
		XSecure_SetReset(InstancePtr->BaseAddress,
			XSECURE_AES_SOFT_RST_OFFSET);
	}

	return Status;

}

/*****************************************************************************/
/**
 *
 * @brief
 * This function updates the GCM tag for the encrypted data.
 *
 * @param	InstancePtr	Pointer to the XSecure_Aes instance.

 * @param	GcmTagAddr	Address to the buffer of GCM tag size, where the API
 *		updates GCM tag.
 *
 * @return
 *		- XST_SUCCESS on successful GCM tag updation
 *		- Error code on failure
 *
 ******************************************************************************/
u32 XSecure_AesEncryptFinal(XSecure_Aes *InstancePtr, u64 GcmTagAddr)
{
	u32 Status = (u32)XST_FAILURE;

	/* Assert validates the input arguments */
	Xil_AssertNonvoid(InstancePtr != NULL);
	Xil_AssertNonvoid(GcmTagAddr != 0x00U);
	Xil_AssertNonvoid(InstancePtr->AesState ==
			XSECURE_AES_ENCRYPT_INITIALIZED);

	XSecure_WriteReg(InstancePtr->BaseAddress,
			XSECURE_AES_DATA_SWAP_OFFSET,
			XSECURE_AES_DATA_SWAP_VAL_MASK);

	XSecure_AesPmcDmaCfgEndianness(InstancePtr->PmcDmaPtr,
					XPMCDMA_DST_CHANNEL, XSECURE_ENABLE_BYTE_SWAP);

	/* Configure the SSS for AES. */
	if (InstancePtr->PmcDmaPtr->Config.DeviceId == PMCDMA_0_DEVICE_ID) {
		Status = XSecure_SssAes(&InstancePtr->SssInstance,
				XSECURE_SSS_DMA0, XSECURE_SSS_DMA0);
	}
	else {
		Status = XSecure_SssAes(&InstancePtr->SssInstance,
				XSECURE_SSS_DMA1, XSECURE_SSS_DMA1);
	}
	if (Status != (u32)XST_SUCCESS) {
			goto END;
	}

	XPmcDma_64BitTransfer(InstancePtr->PmcDmaPtr,
			XPMCDMA_DST_CHANNEL,
			(u32)GcmTagAddr, (u32)(GcmTagAddr >> 32U),
			XSECURE_SECURE_GCM_TAG_SIZE / XSECURE_WORD_SIZE, FALSE);
	/* Wait for the DST DMA completion. */
	XPmcDma_WaitForDone(InstancePtr->PmcDmaPtr, XPMCDMA_DST_CHANNEL);

	/* Acknowledge the transfer has completed */
	XPmcDma_IntrClear(InstancePtr->PmcDmaPtr, XPMCDMA_DST_CHANNEL,
					XPMCDMA_IXR_DONE_MASK);

	/* Status Reset */
	Status = (u32)XST_FAILURE;

	/* Wait for AES Decryption completion. */
	Status = XSecure_AesWaitForDone(InstancePtr);

END:
	InstancePtr->AesState = XSECURE_AES_INITIALIZED;

	XSecure_AesPmcDmaCfgEndianness(InstancePtr->PmcDmaPtr,
			XPMCDMA_DST_CHANNEL, XSECURE_DISABLE_BYTE_SWAP);

	XSecure_WriteReg(InstancePtr->BaseAddress,
			XSECURE_AES_DATA_SWAP_OFFSET, XSECURE_DISABLE_BYTE_SWAP);

	XSecure_SetReset(InstancePtr->BaseAddress,
			XSECURE_AES_SOFT_RST_OFFSET);

	return Status;

}

/*****************************************************************************/
/**
 *
 * @brief
 * This function encrypts size (length) number of bytes of the passed in
 * InDataAddr (source) buffer and stores the encrypted data along with its
 * associated 16 byte tag in the OutDataAddr (destination) buffer and
 * GcmTagAddr (buffer) respectively.
 *
 * @param	InstancePtr	Pointer to the XSecure_Aes instance.
 * @param	InDataAddr	Address of the data which needs to be
 *		encrypted.
 * @param	OutDataAddr	Address of output buffer where the encrypted data
 *		to be updated.
 * @param	Size		Size of data to be encrypted in bytes,
 *			whereas number of bytes provided should be multiples of 4.
 *
 * @return
 *		- XST_SUCCESS on successful encryption.
 *		- Error code on failure
 *
 ******************************************************************************/
u32 XSecure_AesEncryptData(XSecure_Aes *InstancePtr, u64 InDataAddr,
			u64 OutDataAddr, u32 Size, u64 GcmTagAddr)
{
	u32 Status = (u32)XST_FAILURE;

	/* Assert validates the input arguments */
	Xil_AssertNonvoid(InstancePtr != NULL);
	Xil_AssertNonvoid((InDataAddr != 0x00U) && (OutDataAddr != 0x00U));
	Xil_AssertNonvoid((Size % XSECURE_WORD_SIZE) == 0x00U);
	Xil_AssertNonvoid(GcmTagAddr != 0x00U);
	Xil_AssertNonvoid(InstancePtr->AesState ==
			XSECURE_AES_ENCRYPT_INITIALIZED);

	/* Update the data to AES engine */
	Status = XSecure_AesEncryptUpdate(InstancePtr, InDataAddr, OutDataAddr,
					Size, TRUE);
	if (Status != XST_SUCCESS) {
		goto END;
	}

	/* Status Reset */
	Status = (u32)XST_FAILURE;

	/* Call Encrypt final */
	Status = XSecure_AesEncryptFinal(InstancePtr, GcmTagAddr);
END:
	return Status;
}

/*****************************************************************************/
/**
 * @brief
 * This function waits for AES engine completes key loading.
 *
 * @param	InstancePtr	Pointer to the XSecure_Aes instance.
 *
 * @return
 *		- XST_SUCCESS if the AES engine completes key loading.
 *		- XST_FAILURE if a timeout has occurred.
 *
 ******************************************************************************/
static u32 XSecure_AesWaitKeyLoad(XSecure_Aes *InstancePtr)
{
	u32 Status = (u32)XST_FAILURE;
	/* Assert validates the input arguments */
	Xil_AssertNonvoid(InstancePtr != NULL);
	Status = Xil_WaitForEvent(((InstancePtr)->BaseAddress +
					XSECURE_AES_STATUS_OFFSET),
			XSECURE_AES_STATUS_KEY_INIT_DONE_MASK,
			XSECURE_AES_STATUS_KEY_INIT_DONE_MASK,
			XSECURE_AES_TIMEOUT_MAX);

	return Status;

}

/*****************************************************************************/
/**
 * @brief
 * This function sets AES engine to update key and IV during decryption of
 * secure header or footer of encrypted partition.
 *
 * @param	InstancePtr	Pointer to the XSecure_Aes instance.
 * @param	Config		- XSECURE_AES_ENABLE_KUP_IV_UPDATE
 *                                to enable KUP and IV update
 *				- XSECURE_AES_DISABLE_KUP_IV_UPDATE
 *				  to disable KUP and IV update
 *
 * @return
 *		- XST_SUCCESS on successful configuration.
 *		- Error code on failure
 *
 ******************************************************************************/
u32 XSecure_AesCfgKupIv(XSecure_Aes *InstancePtr, u32 Config)
{
	u32 Status = (u32)XST_FAILURE;

	/* Assert validates the input arguments */
	Xil_AssertNonvoid(InstancePtr != NULL);
	Xil_AssertNonvoid((Config == XSECURE_AES_DISABLE_KUP_IV_UPDATE) ||
				(Config == XSECURE_AES_ENABLE_KUP_IV_UPDATE));
	Xil_AssertNonvoid(InstancePtr->AesState != XSECURE_AES_UNINITIALIZED);

	if (Config == XSECURE_AES_DISABLE_KUP_IV_UPDATE) {
		XSecure_WriteReg(InstancePtr->BaseAddress,
				XSECURE_AES_KUP_WR_OFFSET, XSECURE_AES_DISABLE_KUP_IV_UPDATE);
	}
	else {
		XSecure_WriteReg(InstancePtr->BaseAddress,
			XSECURE_AES_KUP_WR_OFFSET,
			(XSECURE_AES_KUP_WR_KEY_SAVE_MASK |
			XSECURE_AES_KUP_WR_IV_SAVE_MASK));
	}

	Status = XST_SUCCESS;

	return Status;
}

/*****************************************************************************/
/**
 * @brief
 * This function gives the AES next block length after decryption of PDI block
 *
 * @param	InstancePtr	Pointer to the XSecure_Aes instance.
 * @param	Size		Pointer to a 32 bit variable where next block
 *		length will be updated.
 *
 * @return	XST_SUCCESS on successful configuration.
 *
 ******************************************************************************/
u32 XSecure_AesGetNxtBlkLen(XSecure_Aes *InstancePtr, u32 *Size)
{
	/* Assert validates the input arguments */
	Xil_AssertNonvoid(InstancePtr != NULL);
	Xil_AssertNonvoid(Size != NULL);
	Xil_AssertNonvoid(InstancePtr->AesState !=
			XSECURE_AES_UNINITIALIZED);

	*Size = Xil_Htonl(XSecure_ReadReg(InstancePtr->BaseAddress,
			XSECURE_AES_IV_3_OFFSET)) * XSECURE_WORD_SIZE;

	return XST_SUCCESS;
}

/*****************************************************************************/
/**
 * @brief
 * This function configures and loads AES key from selected key source.
 *
 * @param	InstancePtr	Pointer to the XSecure_Aes instance.
 * @param	KeySrc		Variable is of type XSecure_AesKeySrc
 *			which mentiones the key source to be loaded into AES engine.
 * @param	KeySize		Size of the key selected.
 *
 * @return
 *		- XST_SUCCESS on successful key load
 *		- Error code on failure.
 *
 ******************************************************************************/
static u32 XSecure_AesKeyLoad(XSecure_Aes *InstancePtr,
			XSecure_AesKeySrc KeySrc, XSecure_AesKeySize KeySize)
{
	u32 Status = (u32)XST_FAILURE;

	/* Assert validates the input arguments */
	Xil_AssertNonvoid(InstancePtr != NULL);
	Xil_AssertNonvoid((KeySrc < XSECURE_MAX_KEY_SOURCES) &&
		(KeySrc >= XSECURE_AES_BBRAM_KEY));
	Xil_AssertNonvoid((KeySize == XSECURE_AES_KEY_SIZE_128) ||
			  (KeySize == XSECURE_AES_KEY_SIZE_256));

	/* Load Key Size */
	XSecure_WriteReg(InstancePtr->BaseAddress, XSECURE_AES_KEY_SIZE_OFFSET, KeySize);

	/* AES key source selection */
	XSecure_WriteReg(InstancePtr->BaseAddress,
			XSECURE_AES_KEY_SEL_OFFSET,
			AesKeyLookupTbl[KeySrc].KeySrcSelVal);

	/* Trig loading of key. */
	XSecure_WriteReg(InstancePtr->BaseAddress,
			XSECURE_AES_KEY_LOAD_OFFSET,
			XSECURE_AES_KEY_LOAD_VAL_MASK);

	/* Wait for AES key loading.*/
	Status = XSecure_AesWaitKeyLoad(InstancePtr);

	return Status;
}

/*****************************************************************************/
/**
 * @brief
 * This function waits for AES completion.
 *
 * @param	InstancePtr Pointer to the XSecure_Aes instance.
 *
 * @return
 *		- XST_SUCCESS on successful key load
 *		- Error code on failure.
 *
 ******************************************************************************/
static u32 XSecure_AesWaitForDone(XSecure_Aes *InstancePtr)
{
	u32 Status = (u32)XST_FAILURE;
	/* Assert validates the input arguments */
	Xil_AssertNonvoid(InstancePtr != NULL);

	Status = Xil_WaitForEvent(((InstancePtr)->BaseAddress +
								XSECURE_AES_STATUS_OFFSET),
								XSECURE_AES_STATUS_DONE_MASK,
								XSECURE_AES_STATUS_DONE_MASK,
								XSECURE_AES_TIMEOUT_MAX);
	return Status;
}

/*****************************************************************************/
/**
 * @brief
 * This function waits for AES key decryption completion.
 *
 * @param	InstancePtr Pointer to the XSecure_Aes instance.
 *
 * @return
 *		- XST_SUCCESS on success
 *		- Error code on failure.
 *
 ******************************************************************************/
static u32 XSecure_AesKekWaitForDone(XSecure_Aes *InstancePtr)
{

	u32 Status = (u32)XST_FAILURE;
	/* Assert validates the input arguments */
	Xil_AssertNonvoid(InstancePtr != NULL);

	Status = Xil_WaitForEvent(((InstancePtr)->BaseAddress +
								XSECURE_AES_STATUS_OFFSET),
								XSECURE_AES_STATUS_BLK_KEY_DEC_DONE_MASK,
								XSECURE_AES_STATUS_BLK_KEY_DEC_DONE_MASK,
								XSECURE_AES_TIMEOUT_MAX);
	return Status;
}

/*****************************************************************************/
/**
 * @brief
 * This function zeroizes the selected AES key storage register.
 *
 * @param	InstancePtr	Pointer to the XSecure_Aes instance.
 * @param	KeySrc	Select the key source which needs to be zeroized.
 *
 * @return
 *		- XST_SUCCESS when key zeroization is success.
 *		- Error code on failure
 *
 ******************************************************************************/
u32 XSecure_AesKeyZero(XSecure_Aes *InstancePtr, XSecure_AesKeySrc KeySrc)
{
	u32 Status = (u32)XST_FAILURE;
	u32 Mask;

	/* Assert validates the input arguments */
	Xil_AssertNonvoid(InstancePtr != NULL);
	Xil_AssertNonvoid((KeySrc < XSECURE_MAX_KEY_SOURCES) &&
		(KeySrc >= XSECURE_AES_BBRAM_KEY));
	Xil_AssertNonvoid(InstancePtr->AesState != XSECURE_AES_UNINITIALIZED);

	if (KeySrc == XSECURE_AES_EXPANDED_KEYS) {
		Mask = XSECURE_AES_KEY_CLEAR_AES_KEY_ZEROIZE_MASK;
	}
	else if (AesKeyLookupTbl[KeySrc].KeyClearVal != XSECURE_AES_INVALID_CFG) {
		Mask = AesKeyLookupTbl[KeySrc].KeyClearVal;
	}
	else {
		Status = XST_INVALID_PARAM;
		goto END;
	}

	XSecure_WriteReg(InstancePtr->BaseAddress, XSECURE_AES_KEY_CLEAR_OFFSET,
					 Mask);

	Status = Xil_WaitForEvent(((InstancePtr)->BaseAddress +
				XSECURE_AES_KEY_ZEROED_STATUS_OFFSET),
				Mask,
				Mask,
				XSECURE_AES_TIMEOUT_MAX);

	if (Status != (u32)XST_SUCCESS) {
		Status = XSECURE_AES_KEY_CLEAR_ERROR;
	}

END:
	XSecure_WriteReg(InstancePtr->BaseAddress, XSECURE_AES_KEY_CLEAR_OFFSET,
					XSECURE_AES_KEY_CLEAR_ALL_AES_KEYS);
	return Status;

}

/******************************************************************************/
/**
 *
 * @brief
 * This is a helper function to enable/disable byte swapping feature of PMC DMA
 *
 * @param	InstancePtr 	Pointer to the XPmcDma instance.
 * @param	Channel 	Channel Type - XPMCDMA_SRC_CHANNEL
 *				XPMCDMA_DST_CHANNEL
 * @param	EndianType 	1 : Enable Byte Swapping
 *				0 : Disable Byte Swapping
 *
 * @return	None
 *
 ******************************************************************************/
static void XSecure_AesPmcDmaCfgEndianness(XPmcDma *InstancePtr,
		XPmcDma_Channel Channel,
		u8 EndianType)
{
	XPmcDma_Configure ConfigValues = {0U};

	/* Assert validates the input arguments */
	Xil_AssertVoid(InstancePtr != NULL);

	XPmcDma_GetConfig(InstancePtr, Channel, &ConfigValues);
	ConfigValues.EndianType = EndianType;
	XPmcDma_SetConfig(InstancePtr, Channel, &ConfigValues);
}

/*****************************************************************************/
/**
 * @brief	This function performs known answer test(KAT) on AES engine.
 *
 * @param	InstancePtr	Pointer to the XSecure_Aes instance
 *
 * @return
 *		- XST_SUCCESS when KAT Pass
 *		- Error code on failure
 *
 *****************************************************************************/
u32 XSecure_AesDecryptKat(XSecure_Aes *AesInstance)
{
	volatile u32 Status = (u32)XST_FAILURE;
	u32 Index;

	const u32 Key[XSECURE_KAT_KEY_SIZE_IN_WORDS] =
			  {0xD55455D7U, 0x2B247897U, 0xC4BF1CDU , 0x1A2D14EDU,
		       0x4D3B0A53U, 0xF3C6E1AEU, 0xAFC2447AU, 0x7B534D99U};
	const u32 Iv[XSECURE_KAT_IV_SIZE_IN_WORDS] =
			  {0xCCF8E3B9U, 0x11F11746U, 0xD58C03AFU, 0x00000000U};
	const u32 Message[XSECURE_KAT_MSG_SIZE_IN_WORDS] =
			  {0xF9ECC5AEU, 0x92B9B870U, 0x31299331U, 0xC4182756U};
	const u32 GcmTag[XSECURE_KAT_GCMTAG_SIZE_IN_WORDS] =
			  {0xC3CFB3E5U, 0x49D4FBCAU, 0xD90B2BFCU, 0xC87DBE9BU};
	const u32 Output[XSECURE_KAT_MSG_SIZE_IN_WORDS] =
			  {0x9008CFD4U, 0x3882AA74U, 0xD635531U,  0x6C1C1F47U};
	u32 DstVal[XSECURE_KAT_MSG_SIZE_IN_WORDS];

	/* Write AES key */
	Status = XSecure_AesWriteKey(AesInstance, XSECURE_AES_USER_KEY_7,
			XSECURE_AES_KEY_SIZE_256, (UINTPTR)Key);
	if (Status != (u32)XST_SUCCESS) {
		Status = XSECURE_AES_KAT_WRITE_KEY_FAILED_ERROR;
		goto END;
	}

	Status = (u32)XST_FAILURE;

	Status = XSecure_AesDecryptInit(AesInstance, XSECURE_AES_USER_KEY_7,
			XSECURE_AES_KEY_SIZE_256, (UINTPTR)Iv);
	if (Status != (u32)XST_SUCCESS) {
		Status = XSECURE_AES_KAT_DECRYPT_INIT_FAILED_ERROR;
		goto END;
	}

	Status = (u32)XST_FAILURE;

	Status =  XSecure_AesDecryptData(AesInstance, (UINTPTR)Message,
			(UINTPTR)DstVal, XSECURE_SECURE_GCM_TAG_SIZE, (UINTPTR)GcmTag);
	if (Status != (u32)XST_SUCCESS) {
		Status = XSECURE_AES_KAT_GCM_TAG_MISMATCH_ERROR;
		goto END;
	}

	/* Initialized to error */
	Status = XSECURE_AES_KAT_DATA_MISMATCH_ERROR;
	for (Index = 0U; Index < XSECURE_AES_BUFFER_SIZE; Index++) {
		if (DstVal[Index] != Output[Index]) {
			/* Comparison failure of decrypted data */
			Status = XSECURE_AES_KAT_DATA_MISMATCH_ERROR;
			goto END;
		}
	}

	if (Index == XSECURE_AES_BUFFER_SIZE) {
		Status = (u32)XST_SUCCESS;
	}

END:
	return Status;
}

/*****************************************************************************/
/**
 * @brief	This function performs KAT on AES core with DPACM enabled
 *
 * @param 	AesInstance	InstancePtr	Pointer to the XSecure_Aes instance
 *
 * @param 	KeyPtr	Key Pointer
 *
 * @param 	DataPtr	Data Pointer
 *
 * @param 	OutputPtr	Output where the decrypted data to be stored
 *
 * @return
 *		- Returns the error codes
 *		- Returns XST_SUCCESS on success
 *
 *****************************************************************************/
static u32 XSecure_AesDpaCmDecryptKat(XSecure_Aes *AesInstance,
						const u32 *KeyPtr, const u32 *DataPtr, u32 *OutputPtr)
{
	volatile u32 Status = (u32)XST_FAILURE;
	u32 Index;

	XSecure_ReleaseReset(AesInstance->BaseAddress,
				XSECURE_AES_SOFT_RST_OFFSET);

	/* Configure AES for Encryption */
	XSecure_WriteReg(AesInstance->BaseAddress,
		XSECURE_AES_MODE_OFFSET, XSECURE_AES_MODE_ENC);

	/* Configure AES in split mode */
	XSecure_WriteReg(AesInstance->BaseAddress, XSECURE_AES_SPLIT_CFG_OFFSET,
		(XSECURE_AES_SPLIT_CFG_KEY_SPLIT |
		XSECURE_AES_SPLIT_CFG_DATA_SPLIT));

	/* Write Key mask value */
	for (Index = 0U; Index < XSECURE_AES_KEY_SIZE_256BIT_WORDS; Index++) {
		XSecure_WriteReg(AesInstance->BaseAddress,
			XSECURE_AES_KEY_MASK_INDEX + (u8)(Index * (u8)XSECURE_WORD_SIZE),
			0x0U);
	}

	/* Write AES key */
	Status = XSecure_AesWriteKey(AesInstance, XSECURE_AES_USER_KEY_7,
			XSECURE_AES_KEY_SIZE_256, (UINTPTR)KeyPtr);
	if (Status != XST_SUCCESS) {
		Status = XSECURE_AESDPACM_KAT_WRITE_KEY_FAILED_ERROR;
		goto END;
	}

	Status = (u32)XST_FAILURE;

	Status = XSecure_AesKeyLoad(AesInstance, XSECURE_AES_USER_KEY_7,
			XSECURE_AES_KEY_SIZE_256);
	if (Status != (u32)XST_SUCCESS) {
		Status = XSECURE_AESDPACM_KAT_KEYLOAD_FAILED_ERROR;
		goto END;
	}

	Status = (u32)XST_FAILURE;

	Status = XSecure_SssAes(&AesInstance->SssInstance,
			XSECURE_SSS_DMA0, XSECURE_SSS_DMA0);
	if(Status != (u32)XST_SUCCESS) {
		Status = XSECURE_AESDPACM_SSS_CFG_FAILED_ERROR;
		goto END;
	}

	/* Start the message. */
	XSecure_WriteReg(AesInstance->BaseAddress, XSECURE_AES_START_MSG_OFFSET,
		XSECURE_AES_START_MSG_VAL_MASK);

	XSecure_WriteReg(AesInstance->BaseAddress, XSECURE_AES_DATA_SWAP_OFFSET,
		XSECURE_AES_DATA_SWAP_VAL_MASK);

	/* Enable PMC DMA Src channel for byte swapping.*/
	XSecure_AesPmcDmaCfgEndianness(AesInstance->PmcDmaPtr,
		XPMCDMA_SRC_CHANNEL, XSECURE_AES_DATA_SWAP_VAL_MASK);
	/* Enable PMC DMA Dst channel for byte swapping.*/
	XSecure_AesPmcDmaCfgEndianness(AesInstance->PmcDmaPtr,
		XPMCDMA_DST_CHANNEL, XSECURE_AES_DATA_SWAP_VAL_MASK);

	/* Configure the PMC DMA Tx/Rx for the incoming Block. */
	XPmcDma_Transfer(AesInstance->PmcDmaPtr, XPMCDMA_DST_CHANNEL,
		(UINTPTR)OutputPtr, XSECURE_AES_DMA_SIZE, XSECURE_AES_DMA_LAST_WORD_DISABLE);

	XPmcDma_Transfer(AesInstance->PmcDmaPtr, XPMCDMA_SRC_CHANNEL,
		(UINTPTR)DataPtr, XSECURE_AES_DMA_SIZE, XSECURE_AES_DMA_LAST_WORD_ENABLE);

	XPmcDma_WaitForDone(AesInstance->PmcDmaPtr, XPMCDMA_DST_CHANNEL);

	Status = (u32)XST_FAILURE;

	Status = XSecure_AesWaitForDone(AesInstance);
	if (Status != (u32)XST_SUCCESS) {
		Status = XSECURE_AESDPACM_KAT_FAILED_ERROR;
		goto END;
	}

	XSecure_AesPmcDmaCfgEndianness(AesInstance->PmcDmaPtr,
		XPMCDMA_SRC_CHANNEL, XSECURE_AES_DATA_SWAP_VAL_DISABLE);

	/* Disable PMC DMA Dst channel for byte swapping. */
	XSecure_AesPmcDmaCfgEndianness(AesInstance->PmcDmaPtr,
		XPMCDMA_DST_CHANNEL, XSECURE_AES_DATA_SWAP_VAL_DISABLE);

	XSecure_WriteReg(AesInstance->BaseAddress, XSECURE_AES_DATA_SWAP_OFFSET,
		XSECURE_AES_DATA_SWAP_VAL_DISABLE);

END:
	XPmcDma_IntrClear(AesInstance->PmcDmaPtr, XPMCDMA_DST_CHANNEL,
		XPMCDMA_IXR_DONE_MASK);
	/* Acknowledge the transfer has completed */
	XPmcDma_IntrClear(AesInstance->PmcDmaPtr, XPMCDMA_SRC_CHANNEL,
		XPMCDMA_IXR_DONE_MASK);
	/* Configure AES in split mode */
	XSecure_WriteReg(AesInstance->BaseAddress, XSECURE_AES_SPLIT_CFG_OFFSET,
		XSECURE_AES_SPLIT_CFG_DATA_KEY_DISABLE);
	/* AES reset */
	XSecure_SetReset(AesInstance->BaseAddress, XSECURE_AES_SOFT_RST_OFFSET);

	return Status;
}

/*****************************************************************************/
/**
 * @brief	This function performs known answer test(KAT) on AES engine
 * to confirm DPA counter measures is working fine.
 *
 * @param 	AesInstance	Pointer to the XSecure_Aes instance
 *
 * @return
 *		- XST_SUCCESS when KAT Pass
 *		- Error code on failure
 *
 *****************************************************************************/
u32 XSecure_AesDecryptCmKat(XSecure_Aes *AesInstance)
{
	volatile u32 Status = (u32)XST_FAILURE;
	const u32 Key0[XSECURE_KAT_KEY_SIZE_IN_WORDS] =
							{0x98076956U, 0x4f158c97U, 0x78ba50f2U, 0x5f7663e4U,
                             0x97e60c2fU, 0x1b55a409U, 0xdd3acbd8U, 0xb687a0edU};

	const u32 Data0[XSECURE_KAT_OPER_DATA_SIZE_IN_WORDS] =
							  {0U, 0U, 0U, 0U, 0x86c237cfU, 0xead48ac1U,
                               0xa0a60b3dU, 0U, 0U, 0U, 0U, 0U, 0x2481322dU,
                               0x568dd5a8U, 0xed5e77d0U, 0x881ade93U};

	const u32 Key1[XSECURE_KAT_KEY_SIZE_IN_WORDS] =
							{0x3ba3028aU, 0x84e787dfU, 0xe38a7a5dU, 0x707e72c8U,
                             0x8cd04f4fU, 0x2883201fU, 0xa5b38c2dU, 0xe9deced3U};

	const u32 Data1[XSECURE_KAT_OPER_DATA_SIZE_IN_WORDS] =
							{0x0U, 0x0U, 0x0U, 0x0U, 0x96589f59U, 0x8e961c85U,
                               0x3b3208d8U, 0x0U, 0x0U, 0x0U, 0x0U, 0x0U,
                               0x328bde4aU, 0xfb2367d5U, 0x40ce658fU, 0xc9275e82U};

	const u32 Ct0[XSECURE_KAT_AES_SPLIT_DATA_SIZE] =
							{0x67020a3bU, 0x3adeecf6U, 0x0309b378U, 0x6ecad4ebU};
	const u32 Ct1[XSECURE_KAT_AES_SPLIT_DATA_SIZE] =
							{0x793391cbU, 0x6575906bU, 0x1a424078U, 0x632b0246U};
	const u32 MiC0[XSECURE_KAT_AES_SPLIT_DATA_SIZE] =
							{0x6400d21fU, 0x6363fc09U, 0x06d4f379U, 0x8809ca7eU};
	const u32 MiC1[XSECURE_KAT_AES_SPLIT_DATA_SIZE] =
							{0x3c459ea7U, 0x5a8aad6fU, 0x878e2a4cU, 0x887f1c82U};

	u32 Output0[XSECURE_KAT_OPER_DATA_SIZE_IN_WORDS];
	u32 Output1[XSECURE_KAT_OPER_DATA_SIZE_IN_WORDS];

	u32 *RM0 = &Output0[0U];
	u32 *R0 = &Output0[4U];
	u32 *Mm0 = &Output0[8U];
	u32 *M0 = &Output0[12U];
	u32 *RM1 = &Output1[0U];
	u32 *R1 = &Output1[4U];
	u32 *Mm1 = &Output1[8U];
	u32 *M1 = &Output1[12U];


	/* Test 1 */
	Status = XSecure_AesDpaCmDecryptKat(AesInstance, Key0, Data0, Output0);
	if (Status != XST_SUCCESS) {
		goto END;
	}

	Status = (u32)XST_FAILURE;

	Status = XSecure_AesDpaCmDecryptKat(AesInstance, Key1, Data1, Output1);
	if (Status != XST_SUCCESS) {
		goto END;
	}

	Status = (u32)XST_FAILURE;

	if (((*(RM0) == 0U) && (*(RM0 + 1U) == 0U) && (*(RM0 + 2U) == 0U) &&
				(*(RM0 + 3U) == 0U)) ||
			((RM0[0U] == RM1[0U]) && (RM0[1U] == RM1[1U]) &&
				(RM0[2U] == RM1[2U]) && (RM0[3U] == RM1[3U])) ||
			((RM0[0U] == Mm0[0U]) && (RM0[1U] == Mm0[1U]) &&
				(RM0[2U] == Mm0[2U]) && (RM0[3U] == Mm0[3U])) ||
			((RM0[0U] == Mm1[0U]) && (RM0[1U] == Mm1[1U]) &&
				(RM0[2U] == Mm1[2U]) && (RM0[3U] == Mm1[3U]))) {
		Status = XSECURE_AESDPACM_KAT_CHECK1_FAILED_ERROR;
		goto END;
	}

	if (((*(RM1) == 0U) && (*(RM1 + 1U) == 0U) && (*(RM1 + 2U) == 0U) &&
				(*(RM1 + 3U) == 0U)) ||
			((RM1[0U] == RM0[0U]) && (RM1[1U] == RM0[1U]) &&
				(RM1[2U] == RM0[2U]) && (RM1[3U] == RM0[3U])) ||
			((RM1[0U] == Mm0[0U]) && (RM1[1U] == Mm0[1U]) &&
				(RM1[2U] == Mm0[2U]) && (RM1[3U] == Mm0[3U])) ||
			((RM1[0U] == Mm1[0U]) && (RM1[1U] == Mm1[1]) &&
				(RM1[2U] == Mm1[2U]) && (RM1[3U] == Mm1[3U]))) {
		Status = XSECURE_AESDPACM_KAT_CHECK2_FAILED_ERROR;
		goto END;
	}

	if (((*(Mm0) == 0U) && (*(Mm0 + 1U) == 0U) && (*(Mm0 + 2U) == 0U) &&
				(*(Mm0 + 3U) == 0U)) ||
			((Mm0[0U] == RM0[0U]) && (Mm0[1U] == RM0[1U]) &&
				(Mm0[2U] == RM0[2U]) && (Mm0[3U] == RM0[3U])) ||
			((Mm0[0U] == RM1[0U]) && (Mm0[1U] == RM1[1U]) &&
				(Mm0[2U] == RM1[2U]) && (Mm0[3U] == RM1[3U])) ||
			((Mm0[0U] == Mm1[0U]) && (Mm0[1U] == Mm1[1U]) &&
				(Mm0[2U] == Mm1[2U]) && (Mm0[3U] == Mm1[3U]))) {
		Status = XSECURE_AESDPACM_KAT_CHECK3_FAILED_ERROR;
		goto END;
	}

	if (((*(Mm1) == 0U) && (*(Mm1 + 1U) == 0U) && (*(Mm1 + 2U) == 0U) &&
				(*(Mm1 + 3U) == 0U)) ||
			((Mm1[0U] == RM0[0U]) && (Mm1[1U] == RM0[1U]) &&
				(Mm1[2U] == RM0[2U]) && (Mm1[3U] == RM0[3U])) ||
			((Mm1[0U] == RM1[0U]) && (Mm1[1U] == RM1[1U]) &&
				(Mm1[2U] == RM1[2U]) && (Mm1[3U] == RM1[3U])) ||
			((Mm1[0U] == Mm0[0U]) && (Mm1[1U] == Mm0[1U]) &&
				(Mm1[2U] == Mm0[2U]) && (Mm1[3U] == Mm0[3U]))) {
		Status = XSECURE_AESDPACM_KAT_CHECK4_FAILED_ERROR;
		goto END;
	}

	if ((((R0[0U] ^ RM0[0U]) != Ct0[0U])  || ((R0[1U] ^ RM0[1U]) != Ct0[1U]) ||
		 ((R0[2U] ^ RM0[2U]) != Ct0[2U])  || ((R0[3U] ^ RM0[3U]) != Ct0[3U])) ||
		(((M0[0U] ^ Mm0[0U]) != MiC0[0U]) || ((M0[1U] ^ Mm0[1U]) != MiC0[1U]) ||
		 ((M0[2U] ^ Mm0[2U]) != MiC0[2U]) || ((M0[3U] ^ Mm0[3U]) != MiC0[3U])) ||
		(((R1[0U] ^ RM1[0U]) != Ct1[0U])  || ((R1[1U] ^ RM1[1U]) != Ct1[1U]) ||
		((R1[2U] ^ RM1[2U]) != Ct1[2U])  || ((R1[3U] ^ RM1[3U]) != Ct1[3U])) ||
		(((M1[0U] ^ Mm1[0U]) != MiC1[0U]) || ((M1[1U] ^ Mm1[1U]) != MiC1[1U]) ||
		 ((M1[2U] ^ Mm1[2U]) != MiC1[2U]) || ((M1[3U] ^ Mm1[3U]) != MiC1[3U]))) {
		Status = XSECURE_AESDPACM_KAT_CHECK5_FAILED_ERROR;
		goto END;
	}

	Status = XST_SUCCESS;
END:
	return Status;
}

/*****************************************************************************/
/**
 *
 * @brief
 * This function initializes the AES engine for decryption.
 *
 * @param       InstancePtr     Pointer to the XSecure_Aes instance.
 * @param       KeySrc          Key Source for decryption of the data.
 * @param       KeySize         Size of the AES key to be used for decryption.
 *               - XSECURE_AES_KEY_SIZE_128 for 128 bit key size
 *               - XSECURE_AES_KEY_SIZE_256 for 256 bit key size
 * @param       IvAddr          Address to the buffer holding IV.
 * @param       AesMode         Aes Mode for Encryption or Decryption.
 *
 * @return
 *              - XST_SUCCESS on successful init
 *              - Error code on failure
 *
 ******************************************************************************/
static u32 XSecure_AesEncNDecInit(XSecure_Aes *InstancePtr,XSecure_AesKeySrc KeySrc,
		XSecure_AesKeySize KeySize, u64 IvAddr)
{
	u32 Status = (u32)XST_FAILURE;

	/* Configure the SSS for AES. */
	if (InstancePtr->PmcDmaPtr->Config.DeviceId == PMCDMA_0_DEVICE_ID) {
		Status = XSecure_SssAes(&InstancePtr->SssInstance,
				XSECURE_SSS_DMA0, XSECURE_SSS_DMA0);
	}
	else {
		Status = XSecure_SssAes(&InstancePtr->SssInstance,
				XSECURE_SSS_DMA1, XSECURE_SSS_DMA1);
	}
	if (Status != (u32)XST_SUCCESS) {
		goto END;
	}

	/* Status Reset */
	Status = (u32)XST_FAILURE;

	Status = XSecure_AesKeyLoad(InstancePtr, KeySrc, KeySize);
	if (Status != XST_SUCCESS) {
		goto END;
	}

	XSecure_WriteReg(InstancePtr->BaseAddress,
			XSECURE_AES_DATA_SWAP_OFFSET, XSECURE_AES_DATA_SWAP_VAL_MASK);


	XSecure_AesPmcDmaCfgEndianness(InstancePtr->PmcDmaPtr,
			XPMCDMA_SRC_CHANNEL, XSECURE_ENABLE_BYTE_SWAP);

	/* Start the message. */
	XSecure_WriteReg(InstancePtr->BaseAddress,
			XSECURE_AES_START_MSG_OFFSET,
			XSECURE_AES_START_MSG_VAL_MASK);

	/* Push IV */
	XPmcDma_64BitTransfer(InstancePtr->PmcDmaPtr, XPMCDMA_SRC_CHANNEL,
			(u32)IvAddr, (u32)(IvAddr >> 32U),
			XSECURE_SECURE_GCM_TAG_SIZE / XSECURE_WORD_SIZE, FALSE);

	XPmcDma_WaitForDone(InstancePtr->PmcDmaPtr,
			XPMCDMA_SRC_CHANNEL);

	/* Acknowledge the transfer has completed */
	XPmcDma_IntrClear(InstancePtr->PmcDmaPtr, XPMCDMA_SRC_CHANNEL,
			XPMCDMA_IXR_DONE_MASK);

	XSecure_AesPmcDmaCfgEndianness(InstancePtr->PmcDmaPtr,
			XPMCDMA_SRC_CHANNEL, XSECURE_DISABLE_BYTE_SWAP);
END:
	return Status;

}
